#!/usr/bin/env python
import user
import maker

import os

class OldMake(maker.Make):
  def setupDependencies(self, sourceDB):
    maker.Make.setupDependencies(self, sourceDB)
    #sourceDB.addDependency(os.path.join('ParMETISLib', 'kmerge.c'), 'parmetis.h')
    return

  def setupConfigure(self, framework):
    shouldRun = maker.Make.setupConfigure(self, framework)
    framework.header  = os.path.join('ParMETISLib','configureheader.h')
    framework.cHeader = None
    return shouldRun

  def configure(self, builder):
    import sys

    if 'PETSC_DIR' in self.argDB:
      dir = self.argDB['PETSC_DIR']
    elif 'PETSC_DIR' in os.environ:
      dir = os.environ['PETSC_DIR']
    else:
      raise RuntimeError('Could not find PETSC_DIR for MPI configure')
    sys.path.append(os.path.join(dir, 'python'))
    framework = maker.Make.configure(self, builder)
    self.mpi = framework.require('PETSc.packages.MPI', None)
    self.libraries = framework.require('config.libraries', None)
    self.setCompilers = framework.require('config.setCompilers', None)
    return

  def buildDirectories(self, builder):
    if not self.prefix is None:
      self.incDir = os.path.abspath(os.path.join(self.prefix, 'include'))
      self.libDir = os.path.abspath(os.path.join(self.prefix, 'lib'))
      self.binDir = os.path.abspath(os.path.join(self.prefix, 'bin'))
    else:
      self.incDir = os.path.abspath('include')
      self.libDir = os.path.abspath('lib')
      self.binDir = os.path.abspath('bin')
    for dir in [self.incDir, self.libDir, self.binDir]:
      if not os.path.isdir(dir):
        os.makedirs(dir)
    return

  def buildMetis(self, builder):
    '''Builds the Metis serial partitioning library'''
    builder.loadConfiguration('Metis Library')
    config = builder.pushConfiguration('Metis Library')
    builder.getCompilerObject().includeDirectories.add('METISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getSharedLinkerObject().libraries.update(self.mpi.lib)
    for source in [f for f in os.listdir('METISLib') if os.path.splitext(f)[1] == '.c']:
      self.builder.compile([os.path.join('METISLib', source)])
    self.metisLibrary = os.path.join(self.libDir, 'libmetis.'+self.setCompilers.sharedLibraryExt)
    self.builder.link(config.outputFiles['ELF'], self.metisLibrary, shared = 1)
    builder.popConfiguration()
    builder.saveConfiguration('Metis Library')
    return

  def buildParMetis(self, builder):
    '''Builds the ParMetis parallel partitioning library'''
    builder.loadConfiguration('ParMetis Library')
    config = builder.pushConfiguration('ParMetis Library')
    builder.getCompilerObject().includeDirectories.add('.')
    builder.getCompilerObject().includeDirectories.add('ParMETISLib')
    builder.getCompilerObject().includeDirectories.add('METISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getSharedLinkerObject().libraries.update(self.mpi.lib)
    for source in [f for f in os.listdir('ParMETISLib') if os.path.splitext(f)[1] == '.c']:
      self.builder.compile([os.path.join('ParMETISLib', source)])
    self.parmetisLibrary = os.path.join(self.libDir, 'libparmetis.'+self.setCompilers.sharedLibraryExt)
    self.builder.link(config.outputFiles['ELF'], self.parmetisLibrary, shared = 1)
    builder.popConfiguration()
    builder.saveConfiguration('ParMetis Library')
    return

  def buildPrograms(self, builder):
    '''Builds the test programs'''
    builder.loadConfiguration('Test Binary')
    builder.pushConfiguration('Test Binary')
    builder.getCompilerObject().includeDirectories.add('.')
    builder.getCompilerObject().includeDirectories.add('Programs')
    builder.getCompilerObject().includeDirectories.add('ParMETISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getLinkerObject().libraries.add(self.parmetisLibrary)
    builder.getLinkerObject().libraries.add(self.metisLibrary)
    if self.libraries.math:
      builder.getLinkerObject().libraries.update(self.libraries.math)
    builder.getLinkerObject().libraries.updateDuplicate(self.mpi.lib)
    sources = ['ptest.c', 'io.c', 'adaptgraph.c']
    for source in sources:
      self.builder.compile([os.path.join('Programs', source)])
    self.builder.link([os.path.join('Programs', self.builder.getCompilerTarget(source)) for source in sources], os.path.join(self.binDir, 'ptest'))
    sources = ['mtest.c', 'io.c']
    for source in sources:
      self.builder.compile([os.path.join('Programs', source)])
    self.builder.link([os.path.join('Programs', self.builder.getCompilerTarget(source)) for source in sources], os.path.join(self.binDir, 'mtest'))
    builder.popConfiguration()
    builder.saveConfiguration('Test Binary')
    return

  def updateDependencies(self, sourceDB):
    sourceDB.updateSource('parmetis.h')
    maker.Make.updateDependencies(self, sourceDB)
    return

  def build(self, builder, setupOnly = 0):
    self.buildDirectories(builder)
    self.buildMetis(builder)
    self.buildParMetis(builder)
    self.buildPrograms(builder)
    return

  def install(self, builder, argDB):
    '''Installs the header'''
    import shutil

    maker.Make.install(self, builder, argDB)
    header = os.path.join(self.incDir, 'parmetis.h')
    if os.path.isfile(header):
      os.remove(header)
    shutil.copy('parmetis.h', self.incDir)
    os.chmod(header, 0664)
    return

class Make(maker.BasicMake):
  def setupDependencies(self, sourceDB):
    maker.Make.setupDependencies(self, sourceDB)
    #sourceDB.addDependency(os.path.join('ParMETISLib', 'kmerge.c'), 'parmetis.h')
    return

  def setupConfigure(self, framework):
    shouldRun = maker.BasicMake.setupConfigure(self, framework)
    framework.require('config.packages.MPI', None)
    framework.header  = os.path.join('ParMETISLib', 'configureheader.h')
    framework.cHeader = None
    return shouldRun

  def configure(self, builder):
    framework = maker.BasicMake.configure(self, builder)
    self.mpi = framework.require('config.packages.MPI', None)
    return

  def setupLibraryDirectories(self, builder):
    maker.BasicMake.setupLibraryDirectories(self, builder)
    self.srcDir['C'] = os.path.abspath(os.getcwd())
    return

  def buildMetis(self, builder):
    '''Builds the Metis serial partitioning library'''
    builder.loadConfiguration('Metis Library')
    config = builder.pushConfiguration('Metis Library')
    builder.getCompilerObject().includeDirectories.add('METISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getSharedLinkerObject().libraries.update(self.mpi.lib)
    for source in [f for f in os.listdir('METISLib') if os.path.splitext(f)[1] == '.c']:
      self.builder.compile([os.path.join('METISLib', source)])
    self.metisLibrary = os.path.join(self.libDir, 'libmetis.'+self.setCompilers.sharedLibraryExt)
    self.builder.link(config.outputFiles['ELF'], self.metisLibrary, shared = 1)
    builder.popConfiguration()
    builder.saveConfiguration('Metis Library')
    return

def dylib_ParMetis_Library(maker):
  '''libmetis: METISLib/defs.h METISLib/macros.h METISLib/metis.h METISLib/proto.h METISLib/rename.h METISLib/stdheaders.h METISLib/struct.h METISLib/balance.c METISLib/bucketsort.c METISLib/ccgraph.c METISLib/checkgraph.c METISLib/coarsen.c METISLib/compress.c METISLib/debug.c METISLib/estmem.c METISLib/fm.c METISLib/fortran.c METISLib/frename.c METISLib/graph.c METISLib/initpart.c METISLib/kmetis.c METISLib/kvmetis.c METISLib/kwayfm.c METISLib/kwayrefine.c METISLib/kwayvolfm.c METISLib/kwayvolrefine.c METISLib/match.c METISLib/mbalance2.c METISLib/mbalance.c METISLib/mcoarsen.c METISLib/memory.c METISLib/mesh.c METISLib/meshpart.c METISLib/mfm2.c METISLib/mfm.c METISLib/mincover.c METISLib/minitpart2.c METISLib/minitpart.c METISLib/mkmetis.c METISLib/mkwayfmh.c METISLib/mkwayrefine.c METISLib/mmatch.c METISLib/mmd.c METISLib/mpmetis.c METISLib/mrefine2.c METISLib/mrefine.c METISLib/mutil.c METISLib/myqsort.c METISLib/ometis.c METISLib/parmetis.c METISLib/pmetis.c METISLib/pqueue.c METISLib/refine.c METISLib/separator.c METISLib/sfm.c METISLib/srefine.c METISLib/stat.c METISLib/subdomains.c METISLib/timing.c METISLib/util.c'''
  return (maker.mpi.include+['METISLib'], maker.mpi.lib, [])

  def buildParMetis(self, builder):
    '''Builds the ParMetis parallel partitioning library'''
    builder.loadConfiguration('ParMetis Library')
    config = builder.pushConfiguration('ParMetis Library')
    builder.getCompilerObject().includeDirectories.add('.')
    builder.getCompilerObject().includeDirectories.add('ParMETISLib')
    builder.getCompilerObject().includeDirectories.add('METISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getSharedLinkerObject().libraries.update(self.mpi.lib)
    for source in [f for f in os.listdir('ParMETISLib') if os.path.splitext(f)[1] == '.c']:
      self.builder.compile([os.path.join('ParMETISLib', source)])
    self.parmetisLibrary = os.path.join(self.libDir, 'libparmetis.'+self.setCompilers.sharedLibraryExt)
    self.builder.link(config.outputFiles['ELF'], self.parmetisLibrary, shared = 1)
    builder.popConfiguration()
    builder.saveConfiguration('ParMetis Library')
    return

def dylib_ParMetis_Library2(maker):
  '''libparmetis: parmetis.h ParMETISLib/defs.h ParMETISLib/macros.h ParMETISLib/parmetislib.h ParMETISLib/proto.h ParMETISLib/rename.h ParMETISLib/stdheaders.h ParMETISLib/struct.h ParMETISLib/akwayfm.c ParMETISLib/ametis.c ParMETISLib/backcompat.c ParMETISLib/balancemylink.c ParMETISLib/coarsen.c ParMETISLib/comm.c ParMETISLib/csrmatch.c ParMETISLib/debug.c ParMETISLib/diffutil.c ParMETISLib/fpqueue.c ParMETISLib/frename.c ParMETISLib/gkmetis.c ParMETISLib/grsetup.c ParMETISLib/iidxsort.c ParMETISLib/iintsort.c ParMETISLib/ikeysort.c ParMETISLib/ikeyvalsort.c ParMETISLib/initbalance.c ParMETISLib/initmsection.c ParMETISLib/initpart.c ParMETISLib/kmetis.c ParMETISLib/kwaybalance.c ParMETISLib/kwayfm.c ParMETISLib/kwayrefine.c ParMETISLib/lmatch.c ParMETISLib/match.c ParMETISLib/mdiffusion.c ParMETISLib/memory.c ParMETISLib/mesh.c ParMETISLib/mmetis.c ParMETISLib/move.c ParMETISLib/node_refine.c ParMETISLib/ometis.c ParMETISLib/order.c ParMETISLib/pspases.c ParMETISLib/redomylink.c ParMETISLib/remap.c ParMETISLib/rmetis.c ParMETISLib/selectq.c ParMETISLib/serial.c ParMETISLib/setup.c ParMETISLib/stat.c ParMETISLib/timer.c ParMETISLib/util.c ParMETISLib/wave.c ParMETISLib/weird.c ParMETISLib/xyzpart.c'''
  return (maker.mpi.include+['ParMETISLib'], maker.mpi.lib+[os.path.join(maker.libDir, 'libmetis.a')], [])

  def buildPrograms(self, builder):
    '''Builds the test programs'''
    builder.loadConfiguration('Test Binary')
    builder.pushConfiguration('Test Binary')
    builder.getCompilerObject().includeDirectories.add('.')
    builder.getCompilerObject().includeDirectories.add('Programs')
    builder.getCompilerObject().includeDirectories.add('ParMETISLib')
    builder.getCompilerObject().includeDirectories.update(self.mpi.include)
    builder.getLinkerObject().libraries.add(self.parmetisLibrary)
    builder.getLinkerObject().libraries.add(self.metisLibrary)
    if self.libraries.math:
      builder.getLinkerObject().libraries.update(self.libraries.math)
    builder.getLinkerObject().libraries.updateDuplicate(self.mpi.lib)
    sources = ['ptest.c', 'io.c', 'adaptgraph.c']
    for source in sources:
      self.builder.compile([os.path.join('Programs', source)])
    self.builder.link([os.path.join('Programs', self.builder.getCompilerTarget(source)) for source in sources], os.path.join(self.binDir, 'ptest'))
    sources = ['mtest.c', 'io.c']
    for source in sources:
      self.builder.compile([os.path.join('Programs', source)])
    self.builder.link([os.path.join('Programs', self.builder.getCompilerTarget(source)) for source in sources], os.path.join(self.binDir, 'mtest'))
    builder.popConfiguration()
    builder.saveConfiguration('Test Binary')
    return

def bin_ParMetis_Executable(maker):
  '''ptest: Programs/ptest.c Programs/io.c Programs/adaptgraph.c'''
  return (['Programs', 'ParMETISLib']+maker.mpi.include, [os.path.join(maker.libDir, 'libparmetis.a'), os.path.join(maker.libDir, 'libmetis.a')]+maker.mpi.lib, [])

def bin_ParMetis_Executable2(maker):
  '''mtest: Programs/mtest.c Programs/io.c'''
  return (['Programs', 'ParMETISLib']+maker.mpi.include, [os.path.join(maker.libDir, 'libparmetis.a'), os.path.join(maker.libDir, 'libmetis.a')]+maker.mpi.lib, [])

if __name__ == '__main__':
  Make().run()
